@@ -1,5 +1,6 @@
 // Copyright (C) 2020 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qquickgraphicsconfiguration_p.h"
 #include <QCoreApplication>
@@ -728,7 +729,7 @@ bool QQuickGraphicsConfiguration::isAutomaticPipelineCacheEnabled() const
 
     \since 6.5
 
-    \sa pipelineCacheLoadFile(), setPipelineCacheSaveFile()
+    \sa pipelineCacheLoadFile(), pipelineCacheSaveFile()
  */
 void QQuickGraphicsConfiguration::setPipelineCacheSaveFile(const QString &filename)
 {
