@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qquickpainteditem.h"
 #include <private/qquickpainteditem_p.h>
@@ -62,9 +63,14 @@ public:
     can potentially be slow if the item is large. This render target allows high quality
     anti-aliasing and fast item resizing.
 
-    \value FramebufferObject As of Qt 6.0, this value is ignored.
+    \value FramebufferObject As of Qt 6.9, this value will enable
+    hardware-accelerated painting as long as the rendering API used is OpenGL,
+    otherwise it will be ignored. For versions Qt 6.0 to Qt 6.8, it will be
+    ignored on all rendering APIs. This will usually give better rendering
+    performance, but at the expense of antialiasing quality.
 
-    \value InvertedYFramebufferObject As of Qt 6.0, this value is ignored.
+    \value InvertedYFramebufferObject Same as FramebufferObject, but with the
+    rendering flipped around the X axis.
 
     \sa setRenderTarget()
 */
@@ -87,7 +93,6 @@ QQuickPaintedItemPrivate::QQuickPaintedItemPrivate()
     , fillColor(Qt::transparent)
     , renderTarget(QQuickPaintedItem::Image)
     , opaquePainting(false)
-    , antialiasing(false)
     , mipmap(false)
     , textureProvider(nullptr)
     , node(nullptr)
@@ -177,6 +182,7 @@ void QQuickPaintedItem::setOpaquePainting(bool opaque)
     QQuickItem::update();
 }
 
+//### Qt7: remove the aa functions; they shadow the QQuickItem property
 /*!
     Returns true if antialiased painting is enabled; otherwise, false is returned.
 
@@ -186,8 +192,7 @@ void QQuickPaintedItem::setOpaquePainting(bool opaque)
 */
 bool QQuickPaintedItem::antialiasing() const
 {
-    Q_D(const QQuickPaintedItem);
-    return d->antialiasing;
+    return QQuickItem::antialiasing();
 }
 
 /*!
@@ -199,13 +204,7 @@ bool QQuickPaintedItem::antialiasing() const
 */
 void QQuickPaintedItem::setAntialiasing(bool enable)
 {
-    Q_D(QQuickPaintedItem);
-
-    if (d->antialiasing == enable)
-        return;
-
-    d->antialiasing = enable;
-    update();
+    QQuickItem::setAntialiasing(enable);
 }
 
 /*!
@@ -544,13 +543,13 @@ QSGNode *QQuickPaintedItem::updatePaintNode(QSGNode *oldNode, UpdatePaintNodeDat
         QSize itemSize(qRound(width()), qRound(height()));
         node->setSize(itemSize);
         QSize textureSize = (hasTextureSize ? d->textureSize : itemSize)
-                            * window()->effectiveDevicePixelRatio();
+                            * d->effectiveDevicePixelRatio();
         node->setTextureSize(textureSize);
     }
 
     node->setPreferredRenderTarget(d->renderTarget);
     node->setFastFBOResizing(d->performanceHints & FastFBOResizing);
-    node->setSmoothPainting(d->antialiasing);
+    node->setSmoothPainting(antialiasing());
     node->setLinearFiltering(d->smooth);
     node->setMipmapping(d->mipmap);
     node->setOpaquePainting(d->opaquePainting);
