@@ -1,5 +1,6 @@
 // Copyright (C) 2017 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qquickcontrol_p.h"
 #include "qquickcontrol_p_p.h"
@@ -16,6 +17,7 @@
 #include "qquickpopup_p.h"
 #include "qquickpopupitem_p_p.h"
 #include "qquickapplicationwindow_p.h"
+#include "qquickapplicationwindow_p_p.h"
 #include "qquickdeferredexecute_p_p.h"
 #include "qquickcontentitem_p.h"
 
@@ -694,6 +696,17 @@ bool QQuickControlPrivate::calcHoverEnabled(const QQuickItem *item)
         if (qobject_cast<const QQuickPopupItem *>(p))
             break;
 
+        auto *applicationWindow = qobject_cast<QQuickApplicationWindow *>(p->window());
+        if (applicationWindow) {
+            const auto *applicationWindowPrivate = QQuickApplicationWindowPrivate::get(applicationWindow);
+            if (p == applicationWindowPrivate->control) {
+                // Don't let the next check get hit, because it will return
+                // false since it's a plain QQuickControl. Instead, skip to the
+                // global flags.
+                break;
+            }
+        }
+
         if (QQuickTemplatesUtils::isInteractiveControlType(p)) {
             const QVariant hoverEnabledProperty = p->property("hoverEnabled");
             Q_ASSERT(hoverEnabledProperty.isValid());
