@@ -1,5 +1,6 @@
 // Copyright (C) 2017 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qquickcontrol_p_p.h"
 #include "qquickoverlay_p.h"
@@ -9,6 +10,7 @@
 #include "qquickdrawer_p.h"
 #include "qquickdrawer_p_p.h"
 #include "qquickapplicationwindow_p.h"
+#include <QtGui/qpainterpath.h>
 #include <QtQml/qqmlinfo.h>
 #include <QtQml/qqmlproperty.h>
 #include <QtQml/qqmlcomponent.h>
@@ -19,7 +21,7 @@ QT_BEGIN_NAMESPACE
 /*!
     \qmltype Overlay
     \inherits Item
-//!     \instantiates QQuickOverlay
+//!     \nativetype QQuickOverlay
     \inqmlmodule QtQuick.Controls
     \since 5.10
     \brief A window overlay for popups.
@@ -66,6 +68,11 @@ void QQuickOverlayPrivate::itemGeometryChanged(QQuickItem *, QQuickGeometryChang
     updateGeometry();
 }
 
+void QQuickOverlayPrivate::itemRotationChanged(QQuickItem *)
+{
+    updateGeometry();
+}
+
 bool QQuickOverlayPrivate::startDrag(QEvent *event, const QPointF &pos)
 {
     Q_Q(QQuickOverlay);
@@ -256,40 +263,15 @@ void QQuickOverlayPrivate::setMouseGrabberPopup(QQuickPopup *popup)
 void QQuickOverlayPrivate::updateGeometry()
 {
     Q_Q(QQuickOverlay);
-    if (!window)
+    if (!window || !window->contentItem())
         return;
 
-    QPointF pos;
-    QSizeF size = window->size();
-    qreal rotation = 0;
-
-    switch (window->contentOrientation()) {
-    case Qt::PrimaryOrientation:
-        rotation = this->rotation();
-        Q_FALLTHROUGH();
-    case Qt::PortraitOrientation:
-        size = window->size();
-        break;
-    case Qt::LandscapeOrientation:
-        rotation = 90;
-        pos = QPointF((size.width() - size.height()) / 2, -(size.width() - size.height()) / 2);
-        size.transpose();
-        break;
-    case Qt::InvertedPortraitOrientation:
-        rotation = 180;
-        break;
-    case Qt::InvertedLandscapeOrientation:
-        rotation = 270;
-        pos = QPointF((size.width() - size.height()) / 2, -(size.width() - size.height()) / 2);
-        size.transpose();
-        break;
-    default:
-        break;
-    }
+    const QSizeF size = window->contentItem()->size();
+    const QPointF pos(-(size.width() - window->size().width()) / 2,
+                      -(size.height() - window->size().height()) / 2);
 
     q->setSize(size);
     q->setPosition(pos);
-    q->setRotation(rotation);
 }
 
 QQuickOverlay::QQuickOverlay(QQuickItem *parent)
@@ -309,7 +291,8 @@ QQuickOverlay::QQuickOverlay(QQuickItem *parent)
         QQuickItemPrivate::get(parent)->addItemChangeListener(d, QQuickItemPrivate::Geometry);
         if (QQuickWindow *window = parent->window()) {
             window->installEventFilter(this);
-            QObjectPrivate::connect(window, &QWindow::contentOrientationChanged, d, &QQuickOverlayPrivate::updateGeometry);
+            if (QQuickItem *contentItem = window->contentItem())
+                QQuickItemPrivate::get(contentItem)->addItemChangeListener(d, QQuickItemPrivate::Rotation);
         }
     }
 }
@@ -317,8 +300,13 @@ QQuickOverlay::QQuickOverlay(QQuickItem *parent)
 QQuickOverlay::~QQuickOverlay()
 {
     Q_D(QQuickOverlay);
-    if (QQuickItem *parent = parentItem())
+    if (QQuickItem *parent = parentItem()) {
         QQuickItemPrivate::get(parent)->removeItemChangeListener(d, QQuickItemPrivate::Geometry);
+        if (QQuickWindow *window = parent->window()) {
+            if (QQuickItem *contentItem = window->contentItem())
+                QQuickItemPrivate::get(contentItem)->removeItemChangeListener(d, QQuickItemPrivate::Rotation);
+        }
+    }
 }
 
 QQmlComponent *QQuickOverlay::modal() const
@@ -441,6 +429,18 @@ void QQuickOverlay::wheelEvent(QWheelEvent *event)
 }
 #endif
 
+/*!
+    \internal
+
+    When clicking inside a scene with a single active popup, a few things can happen.
+    The click can be inside the popup item, in which case, we stop filtering to allow
+    normal event handling. Or the click can be outside the popup, in which case,
+    it will normally be inside the overlay, or the popup's dimmer item.
+
+    When dealing with nested popups, the second popup's dimmer (or popupItem if the dimmer is absent)
+    will become an exclusive grabber to the pointerEvent, during childMouseEventFilter.
+    (sometimes the overlay becomes the exclusive grabber instead, why?).
+*/
 bool QQuickOverlay::childMouseEventFilter(QQuickItem *item, QEvent *event)
 {
     Q_D(QQuickOverlay);
@@ -469,6 +469,13 @@ bool QQuickOverlay::childMouseEventFilter(QQuickItem *item, QEvent *event)
             case QEvent::HoverEnter:
             case QEvent::HoverMove:
             case QEvent::HoverLeave:
+                // If the control item has already been hovered, allow the hover leave event
+                // to be processed by the same item for resetting its internal hovered state
+                // instead of filtering it here.
+                if (auto *control = qobject_cast<QQuickControl *>(item)) {
+                    if (control->isHovered() && event->type() == QEvent::HoverLeave)
+                        return false;
+                }
                 handled = d->handleHoverEvent(item, static_cast<QHoverEvent *>(event), popup);
                 break;
 
@@ -488,6 +495,22 @@ bool QQuickOverlay::childMouseEventFilter(QQuickItem *item, QEvent *event)
     return false;
 }
 
+/*!
+    \internal
+
+    The overlay installs itself as an event filter on the window it belongs to.
+    It will filter Touch, Mouse (press and release) and Wheel related events.
+
+    Touch and MousePress events will be passed to the delivery agent for normal event propagation,
+    where they will be filtered by the overlay again in QQuickOverlay::childMouseEventFilter.
+    All opened popups will then be iterated, to check if the event should close the popup or not.
+    Also modality is checked to determine the return type, which will cause the delivery agent to
+    continue normal propagation in this second childMouseEventFilter filtering step.
+
+    The reason for installing the eventFilter is to allow non-modal popups with CloseOnReleaseOutside
+    as the closing policy to close when clicking the overlay. The delivery agent won't send the
+    release event to the overlay when clicking it directly, only the press event.
+*/
 bool QQuickOverlay::eventFilter(QObject *object, QEvent *event)
 {
     Q_D(QQuickOverlay);
@@ -506,8 +529,10 @@ bool QQuickOverlay::eventFilter(QObject *object, QEvent *event)
 
         // allow non-modal popups to close on touch release outside
         if (!d->mouseGrabberPopup) {
-            for (const QTouchEvent::TouchPoint &point : static_cast<QTouchEvent *>(event)->points()) {
+            QTouchEvent *touchEvent = static_cast<QTouchEvent *>(event);
+            for (const QTouchEvent::TouchPoint &point : touchEvent->points()) {
                 if (point.state() == QEventPoint::Released) {
+                    QQuickDeliveryAgentPrivate::translateTouchEvent(touchEvent);
                     if (d->handleRelease(d->window->contentItem(), event, nullptr))
                         break;
                 }
@@ -526,23 +551,28 @@ bool QQuickOverlay::eventFilter(QObject *object, QEvent *event)
         event->accept();
         // Since we eat the event, QQuickWindow::event never sees it to clean up the
         // grabber states. So we have to do so explicitly.
-        if (QQuickWindow *window = parentItem() ? parentItem()->window() : nullptr) {
-            QQuickWindowPrivate *d = QQuickWindowPrivate::get(window);
-            d->clearGrabbers(static_cast<QPointerEvent *>(event));
-        }
+        d->deliveryAgentPrivate()->clearGrabbers(static_cast<QPointerEvent *>(event));
         return true;
 #endif
 
-    case QEvent::MouseButtonPress:
+    case QEvent::MouseButtonPress: {
+        auto *mouseEvent = static_cast<QMouseEvent *>(event);
+        // Don't filter right mouse button clicks, as it prevents ContextMenu from
+        // receiving QContextMenuEvents as long as e.g. a Drawer exists, even if it's not visible.
+        // This does not prevent popups from being closed with the right mouse button,
+        // as mousePressEvent takes care of that.
+        if (mouseEvent->button() == Qt::RightButton)
+            break;
+
 #if QT_CONFIG(quicktemplates2_multitouch)
         // do not emit pressed() twice when mouse events have been synthesized from touch events
-        if (static_cast<QMouseEvent *>(event)->source() == Qt::MouseEventNotSynthesized)
+        if (mouseEvent->source() == Qt::MouseEventNotSynthesized)
 #endif
             emit pressed();
 
         // setup currentEventDeliveryAgent like in QQuickDeliveryAgent::event
         QQuickDeliveryAgentPrivate::currentEventDeliveryAgent = d->deliveryAgent();
-        d->deliveryAgentPrivate()->handleMouseEvent(static_cast<QMouseEvent *>(event));
+        d->deliveryAgentPrivate()->handleMouseEvent(mouseEvent);
         QQuickDeliveryAgentPrivate::currentEventDeliveryAgent = nullptr;
 
         // If a mouse event hasn't been accepted after being delivered, there
@@ -551,11 +581,15 @@ bool QQuickOverlay::eventFilter(QObject *object, QEvent *event)
         // events, to be able to close non-modal popups on release outside.
         event->accept();
         return true;
+    }
+    case QEvent::MouseButtonRelease: {
+        auto *mouseEvent = static_cast<QMouseEvent *>(event);
+        if (mouseEvent->button() == Qt::RightButton)
+            break;
 
-    case QEvent::MouseButtonRelease:
 #if QT_CONFIG(quicktemplates2_multitouch)
         // do not emit released() twice when mouse events have been synthesized from touch events
-        if (static_cast<QMouseEvent *>(event)->source() == Qt::MouseEventNotSynthesized)
+        if (mouseEvent->source() == Qt::MouseEventNotSynthesized)
 #endif
             emit released();
 
@@ -563,7 +597,7 @@ bool QQuickOverlay::eventFilter(QObject *object, QEvent *event)
         if (!d->mouseGrabberPopup)
             d->handleRelease(d->window->contentItem(), event, nullptr);
         break;
-
+    }
 #if QT_CONFIG(wheelevent)
     case QEvent::Wheel: {
         // If the top item in the drawing-order is blocked by a modal popup, then
@@ -652,6 +686,8 @@ void QQuickOverlayAttachedPrivate::setWindow(QQuickWindow *newWindow)
 
     The signal can be attached to any item, popup, or window. When attached to an
     item or a popup, the signal is only emitted if the item or popup is in a window.
+
+    \include qquickoverlay-pressed-released.qdocinc
 */
 
 /*!
@@ -662,6 +698,8 @@ void QQuickOverlayAttachedPrivate::setWindow(QQuickWindow *newWindow)
 
     The signal can be attached to any item, popup, or window. When attached to an
     item or a popup, the signal is only emitted if the item or popup is in a window.
+
+    \include qquickoverlay-pressed-released.qdocinc
 */
 
 QQuickOverlayAttached::QQuickOverlayAttached(QObject *parent)
